import { assign, createMachine } from 'xstate';
import { Platform } from 'react-native';
import { loggerClient } from '../config/axios';
const OS = {
  ios: 'iOS',
  android: 'Android',
  windows: 'Windows',
  macos: 'Mac OS',
  web: 'Web',
  other: 'Other'
};

/**
 * The function fetchDeviceInfo retrieves information about the device, including the brand, model, and
 * operating system version.
 * @returns The function `fetchDeviceInfo` returns a Promise that resolves to an object of type
 * `DeviceInfo`.
 */
const fetchDeviceInfo = () => {
  const iOS = Platform.OS === 'ios';
  if (iOS) {
    const {
      constants,
      isPad
    } = Platform;
    return {
      brand: 'Apple',
      model: isPad ? 'iPad' : 'iPhone',
      os_version: constants.osVersion
    };
  }
  const {
    constants
  } = Platform;
  return {
    brand: constants.Brand,
    model: constants.Model,
    os_version: constants.Fingerprint
  };
};

/**
 * The function `sendLog` sends log data to a server using a POST request.
 * @param {LoggerContext} context - The `context` parameter is an object that contains various
 * information related to logging. Here's a breakdown of the properties:
 * @returns The function `sendLog` returns a `Promise<void>`.
 */
const sendLog = async context => {
  const {
    content
  } = context;
  if (!content) return;
  const payload = {
    environment: context.environment,
    client_uuid: context.clientUUID,
    user_id: context.userID,
    timestamp: new Date().toISOString(),
    platform: OS[Platform.OS] || OS.other,
    device_info: context.deviceInfo,
    sdk: context.sdkInfo
  };
  if (content.event === 'extract') payload.extraction_log = content;else payload.transmission_log = content;
  await loggerClient.post('/', payload);
};
export const LoggerController = createMachine({
  id: 'LoggerController',
  initial: 'waiting',
  predictableActionArguments: true,
  context: {
    clientUUID: '',
    content: null,
    deviceInfo: null,
    environment: 'development',
    sdkInfo: null,
    timestamp: new Date().toISOString(),
    userID: ''
  },
  states: {
    waiting: {
      on: {
        init: {
          target: 'idle',
          actions: ['handleInit']
        }
      }
    },
    idle: {
      on: {
        send: {
          target: 'sending',
          actions: assign({
            content: (_, event) => event.payload
          })
        }
      }
    },
    sending: {
      invoke: {
        id: 'sendLog',
        src: sendLog,
        onDone: {
          target: 'idle'
        },
        onError: {
          target: 'idle',
          actions: assign((context, _) => {
            return {
              ...context
            };
          })
        }
      }
    }
  }
}, {
  actions: {
    handleInit: assign((context, event) => {
      if (event.type !== 'init') return {
        ...context
      };
      const deviceInfo = fetchDeviceInfo();
      return {
        ...context,
        environment: event.payload.environment,
        clientUUID: event.payload.clientUUID,
        userID: event.payload.userID || '',
        sdkInfo: event.payload.sdkInfo,
        deviceInfo
      };
    })
  }
});
//# sourceMappingURL=LoggerController.js.map