import { useMachine } from '@xstate/react';
import { LoggerController } from '../controllers/LoggerController';
import type {
  ExtractionLog,
  LoggerContext,
  TransmissionLog,
} from '../types/LoggerTypes';
import type { LogBoundaryProps, LogSelector } from '../types/LogBoundaryTypes';

type LoggerProps = Partial<LoggerContext> &
  Omit<LoggerContext, 'timestamp' | 'content' | 'deviceInfo' | 'userID'>;

export const useRookLogger = () => {
  const [_, send] = useMachine(LoggerController);

  const initLogger = (props: LoggerProps): void => {
    const { clientUUID, environment, sdkInfo, userID } = props;

    send({
      type: 'init',
      payload: {
        clientUUID,
        environment,
        sdkInfo,
        userID,
      },
    });
  };

  const sendExtractionLog = (content: ExtractionLog): void => {
    send({ type: 'send', payload: content });
  };

  const sendTransmissionLog = (content: TransmissionLog): void => {
    send({ type: 'send', payload: content });
  };

  /**
   * The `logBoundary` function is a TypeScript function that wraps another function and logs the
   * success or failure of the wrapped function.
   * @param props - The `props` parameter is an object that contains the following properties:
   * @returns The function `logBoundary` returns a `Promise` that resolves to the response from the
   * `fun` function.
   */
  const logBoundary = async <T>(props: LogBoundaryProps<T>): Promise<T> => {
    const { fun, logSuccess, payload, logType, dataType } = props;

    try {
      const response = await fun(payload);

      if (logSuccess)
        sendLog({
          dataType,
          logType,
          message: {
            status: 'Success',
            description: logType.event === 'extract' ? '' : 'created',
          },
        });

      return response;
    } catch (error) {
      const copy = logType;

      if (
        (error as Error).message &&
        (error as Error).message.includes('YYYY-MM-DD')
      ) {
        copy.date = new Date().toISOString();
      }

      sendLog({
        dataType,
        logType: copy,
        message: { status: 'Failure', description: `${error}` },
      });
      throw error;
    }
  };

  /**
   * The function `sendLog` sends log messages based on the log type, either extraction or
   * transmission.
   * @param {LogSelector}  - - `dataType`: The type of data being logged.
   */
  const sendLog = ({ dataType, message, logType }: LogSelector): void => {
    if (logType.type === 'extraction')
      sendExtractionLog({
        data_type: dataType,
        data_datetime: logType.date,
        status: message.status,
        failure_reason: message.description ? message.description : undefined,
        event: logType.event,
      });
    else
      sendTransmissionLog({
        data_type: dataType,
        data_datetime: logType.date,
        server_response: message.description,
        event: logType.event,
      });
  };

  return {
    initLogger,
    sendExtractionLog,
    sendTransmissionLog,
    logBoundary,
  };
};
